#!/bin/bash

basedir=`dirname "$0"`
version="1.0.14"
this=`basename "$0"`

usage () {
   cat <<EOF

Usage:
  $this [options]

  Current version of C++TESK Testing ToolKit

  Options:
    --new                     - find what is new in C++TESK $version
    --log                     - show the complete C++TESK change log
    --version | -v            - show current version of the toolkit and its components (default)
    --help | -h               - show this message

EOF
}

version () {
  if [ -f $basedir/../Version ]
  then
    echo
    tput setaf 3
    cat $basedir/../Version
    tput sgr0
    echo
    echo Use --new option to find what is new in C++TESK Testing ToolKit $version
    echo
  else
    tput setaf 1
    echo "[ERROR] File \"Version\" is missing!"
    tput sgr0
  fi
}

new_info () {
  if [ -f $basedir/../ChangeLog ]
  then
    echo
    tput setaf 3
    head $basedir/../ChangeLog
    tput sgr0
    echo
    echo Use --log option to get the complete C++TESK Testing ToolKit change log
    echo
  else
    tput setaf 1
    echo "[ERROR] File \"ChangeLog\" is missing!"
    tput sgr0
  fi
}

all_info () {
  if [ -f $basedir/../ChangeLog ]
  then
    echo
    tput setaf 3
    cat $basedir/../ChangeLog
    tput sgr0
    echo
  else
    tput setaf 1
    echo "[ERROR] File \"ChangeLog\" is missing!"
    tput sgr0
  fi
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --version | -v )
      version
      exit 0
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    --new )
      new_info
      exit 0
    ;;
    --log )
      all_info
      exit 0
    ;;
    --help | -h | * )
      usage
      exit 0
    ;;
  esac
  shift
done

version

