/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * PROJECT:     CTesK
 * FILE:        dfsm.cpp
 * AUTHOR:      Nikolay Pakulin (npak@ispras.ru)
 * CREATION:    12 Nov 2010
 * DESCRIPTION: Implementation of wrappers to call DFSM methods specified in C++.
 * VERSION AND LIST OF CHANGES:
 */
#include <atl/unit.h>
#include <ts/cpp_iface.hpp>

#include <ts/scenario.hpp>
#include <ts/dfsm.h>
#include <ts/dfsm.hpp>
#include <ts/ndfsm.h>
#include <ts/ndfsm.hpp>
#include <ts/engine.h>

#include <sstream>
#include <cassert>

using namespace cpptesk::ts;

// static int in_cpp_scenario = 0;


static int _scenario_init(TestScenario* dfsm, int argc, char** argv)
{
    return (dfsm->init)? ((CppTestScenario *)dfsm)->init->operator()(argc, argv) : true;
}

static void _scenario_finish(TestScenario* dfsm)
{
    if (dfsm->finish)
    {
        return ((CppTestScenario *)dfsm)->finish->operator()();
    }
}

static Object * _scenario_get_state(TestScenario* dfsm)
{
    return ((CppTestScenario *)dfsm)->getState->operator()();
}

static String * _scenario_get_state_id(TestScenario* dfsm, Object* obj)
{
    return ((CppTestScenario *)dfsm)->getStateId->operator()(obj);
}

static Object * _scenario_save_model_state(TestScenario* dfsm)
{
    return ((CppTestScenario *)dfsm)->saveModelState->operator()();
}

static void _scenario_restore_model_state(TestScenario* dfsm, Object * obj)
{
    return ((CppTestScenario *)dfsm)->restoreModelState->operator()(obj);
}

static int _scenario_is_stationary_state(TestScenario* dfsm)
{
    return ((CppTestScenario *)dfsm)->isStationaryState->operator()();
}

static void _scenario_observe_state(TestScenario* dfsm)
{
    return ((CppTestScenario *)dfsm)->observeState->operator()();
}

#include <ts/scenario.hpp>
String * default_get_state_id::operator() (Object* arg)
{
    String * res = getStateId_Default(arg);
    return res;
}

static void _scenario_set_default_get_state_id(TestScenario* ts)
{
    ((CppTestScenario*)ts)->getStateId = new default_get_state_id();
}

Object * default_get_state::operator() ()
{
    Object * res = singletonState();
    return res;
}

static void _scenario_set_default_get_state(TestScenario* ts)
{
    ((CppTestScenario*)ts)->getState = new default_get_state();
}


static const char * _action_name(ScenarioFunctionID action)
{
    return ((scenario_method_base*)action)->name;
}

static Object * _action_init(ScenarioFunctionID action)
{
    return ((scenario_method_base*)action)->init();
}

static int _action_has_next(ScenarioFunctionID action, Object * par)
{
    return ((scenario_method_base*)action)->has_next(par);
}

static int _action_next(ScenarioFunctionID action, Object * par)
{
    return ((scenario_method_base*)action)->next(par);
}

static int _action_invoke(ScenarioFunctionID action, Object * par)
{
    return ((scenario_method_base*)action)->invoke(par);
}

static Object* _model_state_save(ModelStateManager* manager)
{
    return ((scenario_save_model_state_base*)(manager->saveModelState))->operator()();
}

static void _model_state_restore(ModelStateManager* manager, Object* state)
{
    return ((scenario_restore_model_state_base*)(manager->restoreModelState))->operator()(state);
}
static bool _model_state_is_stationary(ModelStateManager* manager)
{
    return ((scenario_is_stationary_state_base*)(manager->isStationaryState))->operator()();
}
static void _model_state_observe(ModelStateManager* manager)
{
    return ((scenario_observe_state_base*)(manager->observeState))->operator()();
}

static Cpp_Iface _ICpp =
{
    // Scenario_Iface
    {
        &_scenario_init, &_scenario_finish,
        &_scenario_get_state, &_scenario_get_state_id,
        &_scenario_save_model_state, &_scenario_restore_model_state,
        &_scenario_is_stationary_state, &_scenario_observe_state,

        &_scenario_set_default_get_state,
        &_scenario_set_default_get_state_id,
    },
    //Action_Iface
    {
        &_action_name, &_action_init, &_action_has_next, &_action_next, &_action_invoke
    },
	// ModelStateManager_Iface
    {
        &_model_state_save, &_model_state_restore, &_model_state_is_stationary, &_model_state_observe
    },
    // ArcSymbol_Iface
    {
        0,0,0
    }
};

bool cpptesk::ts::start_dfsm(int argc, char** argv, CppTestScenario* td)
{
    enable_cplusplus_mode(1);
    return ::start_dfsm(argc, argv, (TestScenario*)td);
}

bool cpptesk::ts::start_ndfsm(int argc, char** argv, CppTestScenario* td)
{
    enable_cplusplus_mode(1);
    return ::start_ndfsm(argc, argv, (TestScenario*)td);
}

int cpptesk::ts::is_cplusplus_mode()
{
    return ::is_cplusplus_mode();
}

int cpptesk::ts::enable_cplusplus_mode(int enable)
{

    int result = cpptesk::ts::is_cplusplus_mode();
    if (enable)
    {
        ::set_cplusplus_iface(&_ICpp);
    }
    else
    {
        ::set_cplusplus_iface(0);
    }
    return result;
}

namespace cpptesk
{
namespace atl
{
template<>
const Type CppObject<IntCtx>::type = CPPTESK_TYPE_CONSTANT_INIT(IntCtx, "I", &type_Object);
template<>
const Type CppObject<Int2Ctx>::type = CPPTESK_TYPE_CONSTANT_INIT(Int2Ctx, "I", &type_Object);
template<>
const Type CppObject<Int3Ctx>::type = CPPTESK_TYPE_CONSTANT_INIT(Int3Ctx, "I", &type_Object);
template<>
const Type CppObject<IntAbcCtx>::type = CPPTESK_TYPE_CONSTANT_INIT(IntAbcCtx, "I", &type_Object);
}
}

/////////////////////
//
// Scenario class
//
////////////////////

//
// Placeholders
//

const InitMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual::init = 0;
const FinishMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual::finish = 0;
const GetStateMethod<ScenarioBase::UseVirtual,Object*>::type ScenarioBase::UseVirtual::get_state = 0;
const GetStateIdMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual:: get_state_id = 0;
const ObserveModelStateMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual::observe_model_state = 0;
const IsStationaryStateMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual::is_stationary_model_state = 0;
const SaveModelStateMethod<ScenarioBase::UseVirtual, Object*>::type ScenarioBase::UseVirtual::save_model_state = 0;
const RestoreModelStateMethod<ScenarioBase::UseVirtual>::type ScenarioBase::UseVirtual::restore_model_state = 0;

const InitMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod::init = 0;
const FinishMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod::finish = 0;
const GetStateMethod<ScenarioBase::NoMethod,Object*>::type ScenarioBase::NoMethod::get_state = 0;
const GetStateIdMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod:: get_state_id = 0;
const ObserveModelStateMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod::observe_model_state = 0;
const IsStationaryStateMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod::is_stationary_model_state = 0;
const SaveModelStateMethod<ScenarioBase::NoMethod, Object*>::type ScenarioBase::NoMethod::save_model_state = 0;
const RestoreModelStateMethod<ScenarioBase::NoMethod>::type ScenarioBase::NoMethod::restore_model_state = 0;

//
// Functors to call virtual methods
//
struct CallScenarioInit : public scenario_init_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioInit(ScenarioBase * obj) : obj(obj) {}
    bool operator() (int argc, char ** argv)
    {
        return obj->init(argc, argv);
    }
};

struct CallScenarioFinish : public scenario_finish_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioFinish(ScenarioBase * obj) : obj(obj) {}
    void operator() ()
    {
        obj->finish();
    }
};

struct CallScenarioGetState : public scenario_get_state_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioGetState(ScenarioBase * obj) : obj(obj) {}
    Object* operator() ()
    {
        return obj->get_state_object();
    }
};

struct CallScenarioGetStateId : public scenario_get_state_id_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioGetStateId(ScenarioBase * obj) : obj(obj) {}
    String* operator() (Object* arg)
    {
        return obj->get_state_id(arg);
    }
};

struct CallScenarioIsStationary : public scenario_is_stationary_state_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioIsStationary(ScenarioBase * obj) : obj(obj) {}
    bool operator() ()
    {
        return obj->is_stationary_model_state();
    }
};

struct CallScenarioRestoreState : public scenario_restore_model_state_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioRestoreState(ScenarioBase * obj) : obj(obj) {}
    void operator() (Object* arg)
    {
        obj->restore_model_state(arg);
    }
};

struct CallScenarioObserveState : public scenario_observe_state_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioObserveState(ScenarioBase * obj) : obj(obj) {}
    void operator() ()
    {
        obj->observe_model_state();
    }
};

struct CallScenarioSaveModelState : public scenario_save_model_state_base
{
private:
    ScenarioBase * obj;
public:
    CallScenarioSaveModelState(ScenarioBase * obj) : obj(obj) {}
    Object* operator() ()
    {
        return obj->save_model_state_object();
    }
};

//
// ScenarioBase
//
ScenarioBase::ScenarioBase() : _scenario_valid(false), _name(0), _scenario_methods(), _init(0), _finish(0), _get_state(0), _get_state_id(0),
    _is_stationary(0), _observe_state(0), _restore_state(0), _save_model_state(0)
{
    ::memset(&_dfsm, 0, sizeof(_dfsm));
    setup("C++TEST Test Scenario");
}

ScenarioBase::ScenarioBase(const char * name) : _scenario_valid(false), _name(name), _scenario_methods(), _init(0), _finish(0), _get_state(0), _get_state_id(0),
    _is_stationary(0), _observe_state(0), _restore_state(0), _save_model_state(0)
{
    ::memset(&_dfsm, 0, sizeof(_dfsm));
    setup(name);
}

ScenarioBase::~ScenarioBase()
{
    if (_dfsm.actions)
    {
        delete[] _dfsm.actions;
    }
    remove_init();
    remove_finish();
    remove_get_state();
    remove_get_state_id();
    remove_is_stationary_state();
    remove_observe_state();
    remove_save_model_state();
    remove_restore_model_state();

    reset_scenario_methods();
}

//
// ScenarioBase default methods
//

bool ScenarioBase::init(int argc, char ** argv) { return true; }
void ScenarioBase::finish() {}
void ScenarioBase::observe_model_state() {}
bool ScenarioBase::is_stationary_model_state() { return true; }
Object* ScenarioBase::get_state_object() { return ::create_Unit(); }
String* ScenarioBase::get_state_id(Object* arg) { return ::toString_Object(arg); }
void ScenarioBase::restore_model_state(Object*) {}
Object* ScenarioBase::save_model_state_object() { return ::create_Unit(); }

//
// ScenarioBase setup
//
const char* ScenarioBase::set_name(const char * name)
{
    // ,    NULL
    assert(name);
    invalidate();
    const char * result = _name;
    _name = name;
    return result;
}

void ScenarioBase::reset_scenario_methods()
{
    for (Actions::iterator i = _scenario_methods.begin(); i != _scenario_methods.end(); ++i)
    {
        delete (*i);
    }
    _scenario_methods.clear();
}

void ScenarioBase::setup(const char * name)
{
    invalidate();
    _name = name;
    set_init(UseVirtual::init);
    set_finish(UseVirtual::finish);
    set_get_state(UseVirtual::get_state);
    set_get_state_id(UseVirtual::get_state_id);
    set_is_stationary_state(NoMethod::is_stationary_model_state);
    set_observe_state(NoMethod::observe_model_state);
    set_save_model_state(NoMethod::save_model_state);
    set_restore_model_state(NoMethod::restore_model_state);
}

CppTestScenario * ScenarioBase::getTestScenario()
{
    if (!_scenario_valid)
    {
        _dfsm.name = _name;
        if (_dfsm.actions) delete _dfsm.actions;
        _dfsm.actions = new scenario_method_base*[_scenario_methods.size()+1];
        int pos = 0;
        for (Actions::iterator i = _scenario_methods.begin(); i != _scenario_methods.end(); ++i, ++pos)
        {
            _dfsm.actions[pos] = *i;
        }
        _dfsm.actions[pos] = 0;
        _dfsm.init = _init;
        _dfsm.finish = _finish;
        _dfsm.getState = _get_state;
        _dfsm.getStateId = _get_state_id;
        _dfsm.isStationaryState = _is_stationary;
        _dfsm.saveModelState = _save_model_state;
        _dfsm.restoreModelState = _restore_state;
        _dfsm.observeState = _observe_state;
    }
    return &_dfsm;
}

void ScenarioBase::set_default_init()
{
    invalidate();
    _init = new CallScenarioInit(this);
}

void ScenarioBase::set_default_finish()
{
    invalidate();
    _finish = new CallScenarioFinish(this);
}
void ScenarioBase::set_default_get_state()
{
    invalidate();
    _get_state = new CallScenarioGetState(this);
}
void ScenarioBase::set_default_get_state_id()
{
    invalidate();
     _get_state_id = new CallScenarioGetStateId(this);
}
void ScenarioBase::set_default_is_stationary_state()
{
    invalidate();
    _is_stationary = new CallScenarioIsStationary(this);
}
void ScenarioBase::set_default_save_model_state()
{
    invalidate();
    _save_model_state = 0;
}
void ScenarioBase::set_default_restore_model_state()
{
    invalidate();
    _restore_state = new CallScenarioRestoreState(this);
}
void ScenarioBase::set_default_observe_state()
{
    invalidate();
    _observe_state = new CallScenarioObserveState(this);
}

void ScenarioBase::remove_init()
{
    if (_init)
    {
        invalidate();
        delete _init;
        _init = 0;
    }
}
void ScenarioBase::remove_finish()
{
    if (_finish)
    {
        invalidate();
        delete _finish;
        _finish = 0;
    }
}
void ScenarioBase::remove_get_state()
{
    if (_get_state)
    {
        invalidate();
        delete _get_state;
        _get_state = 0;
    }
}
void ScenarioBase::remove_get_state_id()
{
    if (_get_state_id)
    {
        invalidate();
        delete _get_state_id;
        _get_state_id = 0;
    }
}

void ScenarioBase::remove_is_stationary_state()
{
    if (_is_stationary)
    {
        invalidate();
        delete _is_stationary;
        _is_stationary = 0;
    }
}

void ScenarioBase::remove_save_model_state()
{
    if (_save_model_state)
    {
        invalidate();
        delete _save_model_state;
        _save_model_state = 0;
    }
}

void ScenarioBase::remove_restore_model_state()
{
    if (_restore_state)
    {
        invalidate();
        delete _restore_state;
        _restore_state = 0;
    }
}
void ScenarioBase::remove_observe_state()
{
    if (_observe_state)
    {
        invalidate();
        delete _observe_state;
        _observe_state = 0;
    }
}
