#pragma once
// #define CPPTESK_SHORT_SHORT_NAMES
#include <ts/coverage.hpp>
#include <tracer/tracer.hpp>

struct AccountModel
{
    int balance;
    int limit;

     ENUM_COV(BalanceState, "Balance state",
		((NEGATIVE, "Negative balance"),
		(ZERO, "Zero balance"),
		(POSITIVE,"Positive balance"),
        (OVERDRAFT,"Overdraft"),
        (LIMIT_REACHED,"Limit reached"))
		);

	ALIAS_COV(DepositStart, "Balance before deposit", BalanceState);
	CPPTESK_DEFINE_ALIAS_COVERAGE(WithdrawStart, "Balance before withdraw", BalanceState);
	CPPTESK_DEFINE_ALIAS_COVERAGE(DepositEnd, "Balance after deposit", BalanceState);
	CPPTESK_DEFINE_ALIAS_COVERAGE(WithdrawEnd, "Balance after withdraw", BalanceState);

	COMP_COV(Deposit, "Deposit", DepositStart, DepositEnd);
//	CPPTESK_DEFINE_COMPOSED_COVERAGE_EXLUDING(Deposit, "Deposit", DepositStart, DepositEnd,
//		({DepositStart::ZERO, DepositEnd::NEGATIVE},
//         {DepositStart::ZERO, DepositEnd::LIMIT_REACHED},
//         {DepositStart::ZERO, DepositEnd::OVERDRAFT},
//         {DepositStart::POSITIVE, DepositEnd::NEGATIVE},
//         {DepositStart::POSITIVE, DepositEnd::ZERO},
//         {DepositStart::POSITIVE, DepositEnd::LIMIT_REACHED},
//         {DepositStart::POSITIVE, DepositEnd::OVERDRAFT},
//         {DepositStart::NEGATIVE, DepositEnd::LIMIT_REACHED},
//         {DepositStart::NEGATIVE, DepositEnd::OVERDRAFT},
//         {DepositStart::LIMIT_REACHED, DepositEnd::OVERDRAFT}
//        ));

	COMP_COV_EXCL(Withdraw, "Withdraw", WithdrawStart, WithdrawEnd,
		({WithdrawStart::ZERO, WithdrawEnd::POSITIVE},
         {WithdrawStart::NEGATIVE, WithdrawEnd::POSITIVE},
         {WithdrawStart::NEGATIVE, WithdrawEnd::ZERO},
		 {WithdrawStart::LIMIT_REACHED, WithdrawEnd::POSITIVE},
		 {WithdrawStart::LIMIT_REACHED, WithdrawEnd::ZERO},
		 {WithdrawStart::LIMIT_REACHED, WithdrawEnd::NEGATIVE},
		 {WithdrawStart::OVERDRAFT, WithdrawEnd::POSITIVE},
		 {WithdrawStart::OVERDRAFT, WithdrawEnd::ZERO},
		 {WithdrawStart::OVERDRAFT, WithdrawEnd::NEGATIVE},
		 {WithdrawStart::OVERDRAFT, WithdrawEnd::LIMIT_REACHED}
	));

	cpptesk::tracer::coverage::CoverageTracker coverage;

    DepositStart cov_DepositStart() const;
    DepositEnd cov_DepositEnd(int sum) const;
    WithdrawStart cov_WithdrawStart() const;
    WithdrawEnd cov_WithdrawEnd(int sum) const;

    Deposit cov_Deposit(int sum) const;
    Withdraw cov_Withdraw(int sum) const;

    bool pre_deposit(int sum) const;
    void cov_deposit(int sum) ;
    bool post_deposit(int pre_balance, int sum, int result) ;

    bool pre_withdraw(int sum) const;
    void cov_withdraw(int sum) ;
    bool post_withdraw(int pre_balance, int sum, int result) ;

    bool post_get_balance(int pre_balance, int result);

    AccountModel(void) ;
    AccountModel(int limit) ;
    ~AccountModel(void);
};

