/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Reaction arbiters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/message.hpp>
#include <hw/process.hpp>
#include <hw/unused.hpp>

#include <algorithm>
#include <map>
#include <vector>

namespace cpptesk {
namespace hw {

#define CPPTESK_TRIVIAL_ARBITER ::cpptesk::hw::ModelBasedArbiter

#ifdef HW_SHORT_NAMES
    #define TRIVIAL_ARBITER CPPTESK_TRIVIAL_ARBITER
#endif // HW_SHORT_NAMES

////////////////////////////////////////////////////////////////////////////////
// Default Reaction Arbiter
////////////////////////////////////////////////////////////////////////////////

class Arbiter
{
public:
    static Arbiter* createDefaultArbiter();

    Arbiter() {}
    virtual ~Arbiter() {}

    virtual void appendRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(process);
        CPPTESK_UNUSED_ARG(message);
    }

    virtual bool chooseRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(process);
        CPPTESK_UNUSED_ARG(message);

        return true;
    }

    virtual void removeRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(process);
        CPPTESK_UNUSED_ARG(message);
    }
    
    virtual bool isModelBased() const { return true; }

    virtual bool preChooseRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(process);
        CPPTESK_UNUSED_ARG(message);

        return true;
    }
};

class ModelBasedArbiter : public Arbiter
{
public:
    ModelBasedArbiter() {}
    virtual ~ModelBasedArbiter() {}
};

}} // namespace cpptesk::hw

