/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru), Mikhail Chupilko (chupilko@ispras.ru)
// SYNOPSIS: Reaction arbiters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/arbiter/base.hpp>

namespace cpptesk {
namespace hw {

////////////////////////////////////////////////////////////////////////////////
// Interface Ordering Reaction Matcher
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_INPUT_FIFO_ARBITER ::cpptesk::hw::InterfaceOrderingArbiter

#ifdef HW_SHORT_NAMES
    #define INPUT_FIFO_ARBITER CPPTESK_INPUT_FIFO_ARBITER
#endif // HW_SHORT_NAMES

class InterfaceOrderingArbiter: public ModelBasedArbiter
{
public:
    InterfaceOrderingArbiter() {}
    virtual ~InterfaceOrderingArbiter() {}

    virtual void appendRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(message);

        std::vector<Process*> &processes = requests[process.input_iface];
        processes.push_back((Process*)&process);
    }

    virtual bool chooseRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(message);

        const std::vector<Process*> processes = requests.find(process.input_iface)->second;
        CPPTESK_ASSERTION(!processes.empty(), "There are no requests to choose");
        
        return processes[0] == &process;
    }

    virtual void removeRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(message);

        std::vector<Process*> &processes = requests[process.input_iface];
        CPPTESK_ASSERTION(!processes.empty(), "There are no requests to remove");

        for(std::vector<Process*>::iterator i = processes.begin(); i != processes.end(); i++)
            { if(*i == &process) { processes.erase(i); break; } }
    }

protected:
    typedef std::vector<Process*> RequestVector;
    std::map<Interface*, RequestVector> requests;
};

}} // namespace cpptesk::hw

