/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Debug printing facilities 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/stack.hpp>

#include <stdio.h>

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

///////////////////////////////////////////////////////////////////////////////
// Debug
///////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_DEBUG_BUFFER_SIZE 1024

#define CPPTESK_DEBUG_STYLE(fg_color, bg_color) \
    (::cpptesk::hw::debug_style_t){fg_color, bg_color}

#define CPPTESK_DEBUG_STREAM() std::cout

#ifdef HW_SHORT_NAMES
    #define DEBUG_STYLE(fg_color, bg_color) CPPTESK_DEBUG_STYLE(fg_color, bg_color)
    #define DEBUG_STREAM()                  CPPTESK_DEBUG_STREAM()
#endif // HW_SHORT_NAMES

#define CPPTESK_COLORED_DEBUG_PRINT(debug_level, fg_color, bg_color, debug_message) \
    do { \
        /* Dump process stack if an error or warning occured */ \
        bool __cpptesk_dump_stack = debug_level == ::cpptesk::hw::DEBUG_ERROR || \
                                    debug_level == ::cpptesk::hw::DEBUG_WARNING; \
        std::stringstream __cpptesk_buffer; \
        __cpptesk_buffer << debug_message; \
        \
        if(__cpptesk_dump_stack && __cpptesk_inside_process && __cpptesk_process_stack) { \
            __cpptesk_buffer << std::endl << *((::cpptesk::hw::process_stack_t*)__cpptesk_process_stack); \
        } \
        ::cpptesk::hw::Debug::print(debug_level, __cpptesk_buffer.str(), fg_color, bg_color); \
    } while(false)

#define CPPTESK_DEBUG_PRINT(debug_level, debug_message) \
    CPPTESK_COLORED_DEBUG_PRINT(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message)

#define CPPTESK_COLORED_DEBUG_PRINTF(debug_level, fg_color, bg_color, debug_message, ...) \
    do { \
        char __cpptesk_buffer[__CPPTESK_DEBUG_BUFFER_SIZE]; \
        if(snprintf(__cpptesk_buffer, __CPPTESK_DEBUG_BUFFER_SIZE, debug_message, __VA_ARGS__) >= __CPPTESK_DEBUG_BUFFER_SIZE) { \
            CPPTESK_ASSERTION(false, "Too long debug message"); \
        } else { \
            ::cpptesk::hw::Debug::print(debug_level, __cpptesk_buffer, fg_color, bg_color); \
        } \
    } while(false)

#define CPPTESK_DEBUG_PRINTF(debug_level, debug_message, ...) \
    CPPTESK_COLORED_DEBUG_PRINTF(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message, __VA_ARGS__)

#define __CPPTESK_COLORED_DEBUG_PRINT_FILE_LINE(debug_level, fg_color, bg_color, debug_message, file, line) \
    CPPTESK_COLORED_DEBUG_PRINT(debug_level, fg_color, bg_color, debug_message << " (" << file << ":" << std::dec << line << ")")

#define __CPPTESK_DEBUG_PRINT_FILE_LINE(debug_level, debug_message, file, line) \
    __CPPTESK_COLORED_DEBUG_PRINT_FILE_LINE(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message, file, line)

#define __CPPTESK_COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, fg_color, bg_color, debug_message, file, line, ...) \
    CPPTESK_COLORED_DEBUG_PRINTF(debug_level, fg_color, bg_color, debug_message " (%s:%d)", __VA_ARGS__, file, line)

#define __CPPTESK_DEBUG_PRINTF_FILE_LINE(debug_level, debug_message, file, line, ...) \
    __CPPTESK_COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message, file, line, __VA_ARGS__)

#define CPPTESK_COLORED_DEBUG_PRINT_FILE_LINE(debug_level, fg_color, bg_color, debug_message) \
    __CPPTESK_COLORED_DEBUG_PRINT_FILE_LINE(debug_level, fg_color, bg_color, debug_message, __FILE__, __LINE__)

#define CPPTESK_DEBUG_PRINT_FILE_LINE(debug_level, debug_message) \
    CPPTESK_COLORED_DEBUG_PRINT_FILE_LINE(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message)

#define CPPTESK_COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, fg_color, bg_color, debug_message, ...) \
    __CPPTESK_COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, fg_color, bg_color, debug_message, __FILE__, __LINE__, __VA_ARGS__)

#define CPPTESK_DEBUG_PRINTF_FILE_LINE(debug_level, debug_message, ...) \
    CPPTESK_COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, ::cpptesk::hw::DEFAULT, ::cpptesk::hw::DEFAULT, debug_message, __VA_ARGS__)

#define CPPTESK_SET_DEBUG_LEVEL(debug_level, debug_color) \
    ::cpptesk::hw::debug(debug_level, debug_color)

#define CPPTESK_SET_DEBUG_STYLE(debug_level, fg_level, bg_level, fg_message, bg_message) \
    ::cpptesk::hw::Debug::setStyle(debug_level, fg_level, bg_level, fg_message, bg_message)

////////////////////////////////////////////////////////////////////////////////
// Debug printing indent
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_SET_DEBUG_INDENT(debug_indent) \
    ::cpptesk::hw::Debug::setIndent(debug_indent)

#define CPPTESK_DEBUG_INDENT_BEGIN \
    ::cpptesk::hw::Debug::incIndent()

#define CPPTESK_START_DEBUG_INDENT() CPPTESK_DEBUG_INDENT_BEGIN

#define CPPTESK_DEBUG_INDENT_END \
    ::cpptesk::hw::Debug::decIndent()

#define CPPTESK_STOP_DEBUG_INDENT() CPPTESK_DEBUG_INDENT_END

////////////////////////////////////////////////////////////////////////////////
// Duplicating debug output to a file
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_SET_DEBUG_LOGFILE(file_name) \
    ::cpptesk::hw::Debug::setLogFile(file_name)

////////////////////////////////////////////////////////////////////////////////
// Debug printing short names
////////////////////////////////////////////////////////////////////////////////

#ifdef HW_SHORT_NAMES
    #define COLORED_DEBUG_PRINT(debug_level, fg_color, bg_color, debug_message)                 CPPTESK_COLORED_DEBUG_PRINT(debug_level, fg_color, bg_color, debug_message)
    #define DEBUG_PRINT(debug_level, debug_message)                                             CPPTESK_DEBUG_PRINT(debug_level, debug_message)
    #define COLORED_DEBUG_PRINTF(debug_level, fg_color, bg_color, debug_message, ...)           CPPTESK_COLORED_DEBUG_PRINTF(debug_level, fg_color, bg_color, debug_message, __VA_ARGS__)
    #define DEBUG_PRINTF(debug_level, debug_message, ...)                                       CPPTESK_DEBUG_PRINTF(debug_level, debug_message, __VA_ARGS__)
    #define COLORED_DEBUG_PRINT_FILE_LINE(debug_level,fg_color, bg_color, debug_message)        CPPTESK_COLORED_PRINT_FILE_LINE(debug_level, fg_color, bg_color, debug_message)
    #define DEBUG_PRINT_FILE_LINE(debug_level, debug_message)                                   CPPTESK_DEBUG_PRINT_FILE_LINE(debug_level, debug_message)
    #define COLORED_DEBUG_PRINTF_FILE_LINE(debug_level, fg_color, bg_color, debug_message, ...) CPPTESK_COLORED_PRINTF_FILE_LINE(debug_level, fg_color, bg_color, debug_message, __VA_ARGS__)
    #define DEBUG_PRINTF_FILE_LINE(debug_level, debug_message, ...)                             CPPTESK_DEBUG_PRINTF(debug_level, debug_message, __VA_ARGS__)
    #define SET_DEBUG_LEVEL(debug_level, debug_color)                                           CPPTESK_SET_DEBUG_LEVEL(debug_level, debug_color)
    #define SET_DEBUG_STYLE(debug_level, fg_level, bg_level, fg_message, bg_message)            CPPTESK_SET_DEBUG_STYLE(debug_level, fg_level, bg_level, fg_message, bg_message)
    #define SET_DEBUG_INDENT()                                                                  CPPTESK_SET_DEBUG_INDENT()
    #define DEBUG_INDENT_BEGIN                                                                  CPPTESK_DEBUG_INDENT_BEGIN
    #define DEBUG_INDENT_END                                                                    CPPTESK_DEBUG_INDENT_END
    #define START_DEBUG_INDENT()                                                                CPPTESK_START_DEBUG_INDENT()
    #define STOP_DEBUG_INDENT()                                                                 CPPTESK_STOP_DEBUG_INDENT()
    #define SET_DEBUG_LOGFILE(file_name)                                                        CPPTESK_SET_DEBUG_LOGFILE(file_name)
#endif // HW_SHORT_NAMES

namespace cpptesk {
namespace hw {

enum debug_level_t
{
    /// Print all messages.
    DEBUG_TOOL     =  5,
    DEBUG_INTERNAL =  5,

    ////////////////////////////////////////////////////////////////////////////
    // Internal Debug Levels
    ////////////////////////////////////////////////////////////////////////////
    DEBUG_ALL      =  4,
    ////////////////////////////////////////////////////////////////////////////
    // External Debug Levels
    ////////////////////////////////////////////////////////////////////////////

    /// Print all events including raw-level ones.
    DEBUG_MORE     =  4,
    DEBUG_DETAILED =  4,
    /// Print basic events only.
    DEBUG_INFO     =  3,
    DEBUG_BASIC    =  3,
    /// Print messages, warnings and errors.
    DEBUG_USER     =  2,
    DEBUG_MESSAGE  =  2,
    /// Print warnings and errors.
    DEBUG_WARN     =  1,
    DEBUG_WARNING  =  1,
    /// Print errors.
    DEBUG_FAIL     =  0,
    DEBUG_ERROR    =  0,
    /// Do not print debug messages.
    DEBUG_NONE     = -1,
};

enum debug_color_t
{
    DEFAULT = -2,
    NORMAL  = -1,
    BLACK   =  0,
    RED     =  1,
    GREEN   =  2,
    YELLOW  =  3,
    BLUE    =  4,
    MAGENTA =  5,
    CYAN    =  6,
    WHITE   =  7,
};

struct debug_style_t
{
    debug_color_t fg_color;
    debug_color_t bg_color;
};

std::ostream& operator << (std::ostream& out, const debug_style_t &style);

class Debug
{
public:
    static void setLevel(debug_level_t debug_level);

    static void enableColoredPrint(bool debug_color);

    static void setStyle(debug_level_t debug_level, debug_color_t fg_level, debug_color_t bg_level,
        debug_color_t fg_message, debug_color_t bg_message);

    /// Returns the old indent value.
    static unsigned setIndent(unsigned debug_indent);

    static void incIndent();
    static void decIndent();

    static void setLogFile(const char* file_name);

    static void print(debug_level_t debug_level, const std::string &debug_message,
        debug_color_t fg_color = DEFAULT, debug_color_t bg_color= DEFAULT);

private:
    struct debug_config_t
    {
        std::string   tag;
        debug_style_t lvl_style;
        debug_style_t msg_style;
    };

private:
    Debug() {}
   ~Debug() {}

private:
    static debug_config_t _debug_config[DEBUG_TOOL + 1];
    static debug_level_t  _debug_level;
    static bool           _debug_color;

    static debug_level_t  _last_debug_level;
    static unsigned       _debug_count;

    static unsigned       _debug_indent;

    /// To duplicate debug output into a file (feature #1900).
    static std::ofstream *_debug_logfile;
};

inline void debug(debug_level_t debug_level = DEBUG_ALL, bool debug_color = true)
    { Debug::setLevel(debug_level); Debug::enableColoredPrint(debug_color); }

}} // namespace cpptesk::hw

